unit uExtractSQL;

interface

uses
  Classes, SqlExpr, SysUtils, Provider, dbClient;

type
  TExtractType = (teTable, teView, teGenerator, teFK,
    teIndex, teColumns, teProc, teTriggers, teDomain,
    teException, teUDF);

TExtractSQL = class(TComponent)
  private
    FSQLConnection: TSQLConnection;
    FLista: TStrings;
    FQuery: TSQLQuery;
    function GetSQLConnection: TSQLConnection;
    procedure SetSQLConnection(const Value: TSQLConnection);
    procedure CreateComponentes;
    procedure Notification(aComponent: TComponent; Operation: TOperation);
    function GetComandoSQL(aType: TExtractType): string;
    function Execute(aType: TExtractType; sTable: string): TStrings;
  protected

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function ExtrairMetadados(aType: TExtractType; sTable: string): TStrings; overload;
    function ExtrairMetadados(aType: TExtractType): TStrings; overload;
  published
    property SQLConnection: TSQLConnection read GetSQLConnection write SetSQLConnection;

  end;

  procedure register;

implementation

uses uComandosSQL;

{ TExtractSQL }

procedure register;
begin
  RegisterComponentsProc('dbexpress', [TExtractSQL]);
end;

procedure TExtractSQL.CreateComponentes;
begin
  FSQLConnection := SQLConnection;
  { caso no tenha sido atribudo um componente SQLConnection emite uma mensagem de erro }
  if FSQLConnection = nil then
    raise Exception.Create('Propriedade SQLConnection no configurada');

  if not Assigned(FQuery) then
  begin
    FQuery := TSQLQuery.Create(nil);
    FQuery.SQLConnection := FSQLConnection
  end;
end;

function TExtractSQL.GetComandoSQL(aType: TExtractType): string;
begin
  { verifica qual o tipo de consulta e retorna o SQL correspondente }
  case aType of
    teTable: Result := TableSQL;
    teView: Result := ViewSQL;
    teGenerator: Result := GeneratorSQL;
    teFK: Result := ForeignNameSQL;
    teIndex: Result := IndexSQL;
    teColumns: Result := ColumnsSQL;
    teProc: Result := ProcSQL;
    teTriggers: Result := TriggerSQL;
    teDomain: Result := DomainSQL;
    teException: Result := ExceptionSQL;
    teUDF: Result := UDFSQL;
    else
      raise Exception.Create('Tipo invlido para a funo GetComandoSQL');
  end;
end;

function TExtractSQL.Execute(aType: TExtractType; sTable: string): TStrings;
var
  aSQL: string;
begin
  { Instancia campos }
  CreateComponentes;
  { retorna o SQL de acordo com o tipo }
  aSQL := GetComandoSQL(aType);
  FQuery.SQL.Clear;
  FQuery.SQL.Add(aSQL);
  { se a tabela no form informada, no precisa de parametro na consulta }
  if (sTable <> '') then
    FQuery.Params[0].AsString := Trim(sTable);
  FQuery.Open;
  FLista.Clear;
  if not FQuery.IsEmpty then
  begin
    { adiciona os metadados na Lista }
    while not FQuery.Eof do
    begin
      FLista.Add(Trim(FQuery.Fields[0].AsString));
      FQuery.Next;
    end;
  end;
  Result := FLista;
end;

function TExtractSQL.ExtrairMetadados(aType: TExtractType; sTable: string): TStrings;
begin
  { verifica se os tipos e parmetros esto corretos e extrai os metadados}
  if (aType = teFK) or (aType = teIndex) or (aType = teColumns) then
  begin
    if (sTable <> '') then
      Result := Execute(aType, sTable)
    else
      raise Exception.Create(' necessrio informar um valor quando escolher a extrao de ndices, colunas ou chaves estrangeiras');
  end
  else
    raise Exception.Create('Valor de aType  invlido para esse mtodo');
end;

function TExtractSQL.ExtrairMetadados(aType: TExtractType): TStrings;
begin
  { extrai os metadados sem parmetros }
  Result := Execute(aType, '');
end;

procedure TExtractSQL.Notification(aComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (aComponent = FSQLConnection) and (Operation = opRemove) then
    FSQLConnection := nil;
end;

constructor TExtractSQL.Create(AOwner: TComponent);
begin
  inherited;
  FLista := TStringList.Create;
end;

destructor TExtractSQL.Destroy;
begin
  FQuery.Free;
  FLista.Free;
  inherited;
end;

function TExtractSQL.GetSQLConnection: TSQLConnection;
begin
  Result := FSQLConnection;
end;

procedure TExtractSQL.SetSQLConnection(const Value: TSQLConnection);
begin
  if FSQLConnection <> Value then
    FSQLConnection := Value;
end;

end.
